<?php
include_once(FORUM_MMOO_DATA_DIR.'_data.php');

class forum_mmoo_data_members extends forum_mmoo_data{
	function __construct(){
		parent::__construct('members');
		
		$this->colNames = array();
		$this->savedAs = null;
		$this->randId = null;
		$this->neccessary = array();
		
		$this->rand_id_length = 15;
		$this->status = array(
			'immediate' => array('flag'=>1, 'text'=>'Activated')
			,'email' => array('flag'=>2, 'text'=>'Mail Sent')
			,'admin' => array('flag'=>3, 'text'=>'Manual')
			,'expired' => array('flag'=>8)
			,'deleted' => array('flag'=>9)
		);
		
		include_once(FORUM_MMOO_DATA_DIR.'data_member_info.php');
		$this->objMemberInfo = new forum_mmoo_data_member_info();
		$this->memberInfo = $this->objMemberInfo->getData();
	}
	
	function setAllFields(){
		$fields = array(
			'status' => array('col_name'=>'status', 'type'=>'char', 'length'=>1)
			, 'rand_id' => array('col_name'=>'rand_id', 'type'=>'char', 'length'=>$this->rand_id_length )
			, 'group_id' => array('col_name'=>'group_id', 'type'=>'int', 'length'=>11, 'index'=>'INDEX')
			, 'message_notify' => array('col_name'=>'message_notify', 'type'=>'char', 'length'=>1, 'default'=>FORUM_MMOO_FLAG_NO)
		);
		$this->setField($fields);
		
		$this->setColNames();
		foreach($this->colNames as $colName){
			$this->setField( array($colName => array('col_name'=>$colName,'type'=>'varchar', 'length'=>256, 'index'=>'FULLTEXT')) );
		}
	}
	
	function setColNames(){
		for($i=1; $i<=FORUM_MMOO_MEMBER_INFO_NUM; $i++){
			$this->colNames[$i] = $this->getColName($i);
		}
	}
	
	function getColName($memberInfoId){
		if( !empty($this->memberInfo[$memberInfoId]['member_col_name']) ){
			return $this->memberInfo[$memberInfoId]['member_col_name'];
		}else{
			return 'value_'.$memberInfoId;
		}
	}
	
	function getWhereAlive( $status='' ){
		switch($status){
			case 'all':
				$where = '';
			break;
			
			case 'strict':
				$where = array('status'=>$this->status['immediate']['flag']);
			break;
			
			default:
				$where = array(
					'status' => array(
						'not in' => array(
							$this->status['deleted']['flag']
							, $this->status['expired']['flag']
						)
					)
				);
			break;
		}
		return $where;
	}
	
	function getMemberNum(){
		$return = array();
		$result = $this->executeSelect( array('group_id','status','COUNT(id) as cnt'), $this->getWhereAlive(), ' GROUP BY group_id, status ' );
		foreach($result as $row){
			$row = $this->decodeFlag('status', 'status', $row);
			$return[ $row['group_id'] ][ $row['status'] ] = $row['cnt'];
		}
		return $return;
	}
	
	function getMemberNumTotal($arrWhere=array(), $strWhere='', $status=''){
		$where = array_merge($this->getWhereAlive($status),$arrWhere);
		$and = empty($where) ? ' WHERE ' : ' AND ';
		$option =  empty($strWhere) ? '' : $and.' ( '.$strWhere.' ) ';
		
		$result = $this->executeSelectOne( array('COUNT(id) as cnt'), $where, $option );
		return $result['cnt'];
	}
	
	function saveData($row){
		include_once(FORUM_MMOO_DATA_DIR.'data_group.php');
		$objGroup = new forum_mmoo_data_group();
		$groupData = $objGroup->getGroupData($row['group_id']);
		
		switch($groupData['allow']){
			case 'immediate':
				$this->savedAs = 'immediate';
				$row['status'] = $this->status['immediate']['flag'];
			break;
			
			case 'email':
				$this->savedAs = 'email';
				$row['status'] = $this->status['email']['flag'];
				$this->randId = wp_generate_password($this->rand_id_length, false, false);
				$row['rand_id'] = $this->randId;
			break;
			
			case 'admin':
				$this->savedAs = 'admin';
				$row['status'] = $this->status['admin']['flag'];
			break;
			
			default:
			return false;
		}
		return $this->executeInsert($row);
	}
	
	function mailConfirm($randId){
		$result = $this->executeSelectOne( array("id", "DATE_FORMAT(updated_time,'%Y%m%d') reg_date"), array('rand_id'=>$randId, 'status'=>$this->status['email']['flag']) );
		if( empty($result) || $result['reg_date']+FORUM_MMOO_MAIL_CONFIRM_LIMIT<date('Ymd') ){
			$set = array( 'status' => $this->status['expired']['flag'] , 'rand_id' => '' );
			$where = array( 'rand_id' => $randId );
			$this->executeUpdate( $set , $where );
			return false;
		}
		
		$set = array( 'status' => $this->status['immediate']['flag'] , 'rand_id' => '' );
		$where = array( 'id' => $result['id'] );
		return $this->executeUpdate( $set, $where );
	}
	
	function adminConfirm($id){
		$set = array( 'status' => $this->status['immediate']['flag'] );
		$where = array( 'id' => $id );
		return $this->executeUpdate( $set, $where );
	}
	
	function adminRecover($id){
		$set = array( 'status' => $this->status['admin']['flag'] );
		$where = array( 'id' => $id );
		return $this->executeUpdate( $set, $where );
	}
	
	function getNeccessary(){
		$checkAlive = $this->objMemberInfo->executeSelectOne( array('alive'), array('member_col_name'=>'login_id') );
		if( $checkAlive['alive']==FORUM_MMOO_FLAG_YES ){
			return array('login_id','password');
		}else{
			return array('mail','password');
		}
	}
	
	function getMemberInfoByLoginData( $forLogin , $status='' ){
		$neccessary = $this->getNeccessary();
		$where = $this->getWhereAlive($status);
		foreach($neccessary as $param){
			if( !isset($forLogin[$param]) ){
				return false;
			}
			$where[$param] = $forLogin[$param];
		}
		
		$result = $this->executeSelectOne('', $where);
		if( empty($result) || empty($result['id']) ){
			return false;
		}
		return $this->decodeFlag('status', 'status', $result);
	}
	
	function getMemberInfoByMail( $mail, $status='' ){
		$where = $this->getWhereAlive($status);
		$where['mail'] = $mail;
		$result = $this->executeSelect('', $where );
		$return = array();
		foreach($result as $row){
			$return[$row['id']] = $this->decodeFlag('status', 'status', $row);
		}
		return $return;
	}
	
	function getMemberInfoById( $id , $status='' ){
		$where = $this->getWhereAlive($status);
		$where['id'] = $id;
		$result = $this->executeSelectOne('', $where );
		return $this->decodeFlag('status', 'status', $result);
	}
	
	function getMemberInfoByArrId( $arrId, $status=''){
		$where = $this->getWhereAlive($status);
		$where['id'] = array('in' => $arrId);
		$result = $this->executeSelect('', $where );
		$return = array();
		foreach($result as $row){
			$return[$row['id']] = $this->decodeFlag('status', 'status', $row);
		}
		return $return;
	}
	
	function getMemberInfoByRandId( $randId , $status='' ){
		$where = $this->getWhereAlive($status);
		$where['rand_id'] = $randId;
		$result = $this->executeSelectOne('', $where );
		return $this->decodeFlag('status', 'status', $result);
	}
	
	function getMemberInfoForPage( $arrWhere=array(), $strWhere='', $status='', $page=1 , $perPage=FORUM_MMOO_DASHBOARD_MEMBER_NUM, $sortCol='', $sortOrder='asc'){
		$where = array_merge($this->getWhereAlive($status),$arrWhere);
		$and = empty($where) ? ' WHERE ' : ' AND ';
		$option =  empty($strWhere) ? '' : $and.' ( '.$strWhere.' ) ';
		
		$arrOrder = array( 
			'group_id ' => 'asc'
			, 'status ' => 'asc'
			, 'id' => 'asc'
		);
		if( !empty($sortCol) ){
			unset($arrOrder[$sortCol]);
			$arrOrder = array_merge(array($sortCol=>$sortOrder), $arrOrder);
		}
		$orderBy = ' ORDER BY ';
		$and = '';
		foreach($arrOrder as $key => $order){
			$orderBy .= $and.$key.' '.$order;
			$and = ' , ';
		}
		
		$limit = $this->makeLimitSql($perPage, $page);
		$result = $this->executeSelect( '', $where, $option.$orderBy.$limit);
		foreach($result as $row){
			$return[$row['id']] = $this->decodeFlag('status', 'status', $row);
		}
		return $return;
	}
	
	function deleteMember($memberId){
		return $this->executeUpdate( array('status'=>$this->status['deleted']['flag']), array('id'=>$memberId) );
	}
	
	function changeMemberGroup($memberId, $groupId){
		return $this->executeUpdate( array('group_id'=>$groupId), array('id'=>$memberId) );
	}
}
?>
