<?php
include_once(FORUM_MMOO_DATA_DIR.'_data.php');

class forum_mmoo_data_message extends forum_mmoo_data{
	function __construct(){
		parent::__construct('message');
		
		$this->from_status = array(
			'draft' => array('flag'=>'1', 'text'=>'Draft')
			, 'outbox' => array('flag'=>'2', 'text'=>'Outbox')
			, 'trash_draft' => array('flag'=>'3', 'text'=>'Trash:Draft')
			, 'trash_outbox' => array('flag'=>'4', 'text'=>'Trash:Outbox')
			, 'never_to_be_seen' => array('flag'=>'9')
		);
		
		$this->to_status = array(
			'new' => array('flag'=>'1', 'text'=>'new')
			, 'read' => array('flag'=>'2', 'text'=>'read')
			, 'trash_new' => array('flag'=>'3', 'text'=>'Trash:new')
			, 'trash_read' => array('flag'=>'4', 'text'=>'Trash:read')
			, 'never_to_be_seen' => array('flag'=>'9')
		);
		
		$this->default_sent_at = '00000000000000';
	}
	
	function setAllFields(){
		$fields = array(
			'from_member' => array('col_name'=>'from_member', 'type'=>'int', 'length'=>11)
			, 'to_member' => array('col_name'=>'to_member', 'type'=>'int', 'length'=>11)
			, 'title' => array('col_name'=>'title', 'type'=>'varchar', 'length'=>256)
			, 'message' => array('col_name'=>'message', 'type'=>'text')
			, 'from_status' => array('col_name'=>'from_status', 'type'=>'char', 'length'=>1, 'default'=>$this->from_status['draft']['flag'])
			, 'to_status' => array('col_name'=>'to_status', 'type'=>'char', 'length'=>1, 'default'=>$this->to_status['new']['flag'])
			, 'sent_at' => array('col_name'=>'sent_at','type'=>'char', 'length'=>14, 'default'=>$this->default_sent_at )
		);
		$this->setField($fields);
	}
	
	function getWhereForBox($memberId, $box){
		$where = array();
		switch( $box ){
			case 'inbox':
				$where = array(
					'to_member' => $memberId
					, 'to_status' => array( 
						'in' => array(
							$this->to_status['new']['flag']
							, $this->to_status['read']['flag']
						)
					)
					, 'sent_at' => array(
						'not' => $this->default_sent_at
					)
				);
			break;
			case 'trash_inbox':
				$where = array(
					'to_member' => $memberId
					, 'to_status' => array(
						'in' => array(
							$this->to_status['trash_new']['flag']
							, $this->to_status['trash_read']['flag']
						)
					)
					, 'sent_at' => array(
						'not' => $this->default_sent_at
					)
				);
			break;
			case 'draft':
			case 'outbox':
			case 'trash_draft':
			case 'trash_outbox':
				$where = array(
					'from_member' => $memberId
					, 'from_status' => $this->from_status[$box]['flag']
				);
			break;
			default:
			break;
		}
		return $where;
	}
	
	function getMessageNum($memberId, $box){
		$where = $this->getWhereForBox($memberId, $box);
		if( empty($where) ){
			return false;
		}
		return $this->getCntRows($where);
	}
	
	function getMessageList($memberId, $box, $page=1, $perPage=FORUM_MMOO_MESSAGE_NUM){
		$this->setAllFields();
		$select = $this->fields;
		unset($select['message']);
		$select = array_keys($select);
		
		$where = $this->getWhereForBox($memberId, $box);
		if( empty($where) ){
			return false;
		}
		
		$limit = $this->makeLimitSql($perPage, $page);
		
		$result = $this->executeSelect( $select , $where, 'order by sent_at desc '.$limit );
		$return = array();
		foreach($result as $row){
			$row = $this->decodeFlag('from_status', 'from_status', $row);
			$return[$row['id']] = $this->decodeFlag('to_status', 'to_status', $row);
		}
		return $return;
	}
	
	function getMessage($messageId){
		$result = $this->executeSelectOne( '', array('id'=>$messageId) );
		$result = $this->decodeFlag('from_status', 'from_status', $result);
		return $this->decodeFlag('to_status', 'to_status', $result);
	}
	
	function newMessage($fromMember, $toMember, $title, $message, $status='draft'){
		$insertRow = array(
			'from_member'=>$fromMember
			, 'to_member'=>$toMember
			, 'title'=>$title
			, 'message'=>$message
			, 'from_status'=>$this->from_status[$status]['flag']
		);
		if( $status == 'outbox' ){
			$insertRow['sent_at'] = $this->now;
		}
		return $this->executeInsert( $insertRow );
	}
	
	function editMessage($messageId, $title, $message ){
		return $this->executeUpdate( array('title'=>$title, 'message'=>$message), array('id'=>$messageId) );
	}
	
	function changeToStatus($messageId, $status){
		$set = array( 'to_status' => $this->to_status[$status]['flag'] );
		return $this->executeUpdate( $set , array('id'=>$messageId) );
	}
	function changeFromStatus($messageId, $status){
		$set = array('from_status'=>$this->from_status[$status]['flag']);
		if( $status == 'outbox' ){
			$set['sent_at'] = $this->now;
		}
		return $this->executeUpdate( $set , array('id'=>$messageId) );
	}
}
?>